--******************
--Brent Ozar Unlimited  2013 All Rights Reserved
--http://BrentOzar.com

--WARNING:
--This script suitable only for test purposes

--TOPIC:	Devs vs DBAs- Perf Tuning Challenge
--PURPOSE:	Find frequently run queries in the plan cache
--******************

use master;
GO



--There's something funny about the query cache....
SELECT query_hash, 
	COUNT(*) AS number_of_plans,
	COUNT(DISTINCT query_plan_hash) as distinct_plans
FROM     sys.dm_exec_query_stats qs
WHERE query_hash <> 0x0000000000000000 /* ignore cursor bits */
GROUP BY qs.[query_hash]
HAVING COUNT(*) > 1
ORDER BY number_of_plans DESC;
GO



--Look for queries grouped by query_hash
--Sort by execution count (descending)
--This only works on SQL Server 2008+
use master;
GO
;WITH    top_queries_by_hash
          AS ( SELECT TOP 25
                        qs.[query_hash] ,
                        COUNT(*) AS number_of_plans ,
                        COUNT(DISTINCT query_plan_hash) as distinct_plans,
                        MAX(plan_handle) AS max_plan_handle ,
                        MAX(qs.plan_generation_num) AS max_plan_generation_number ,
                        MIN(qs.creation_time) AS min_creation_time ,
                        MAX(qs.last_execution_time) AS max_last_execution_time ,
                        SUM(qs.[execution_count]) AS execution_count ,
                        SUM(qs.[total_worker_time]) AS total_worker_time ,
                        SUM(qs.[total_physical_reads]) AS total_physical_reads ,
                        SUM(qs.[total_logical_writes]) AS total_logical_writes ,
                        SUM(qs.[total_logical_reads]) AS total_logical_reads ,
                        SUM(qs.[total_clr_time]) AS total_clr_time ,
                        SUM(qs.[total_elapsed_time]) AS total_elapsed_time
               FROM     sys.dm_exec_query_stats qs
               GROUP BY qs.[query_hash]
               ORDER BY execution_count DESC
             ),
        query_text_and_plans
          AS ( SELECT   tq.query_hash ,
						qs.query_plan_hash,
                        SUBSTRING(st.text, ( qs.statement_start_offset / 2 ) + 1,
                                  ( ( CASE qs.statement_end_offset
                                        WHEN -1 THEN DATALENGTH(st.text)
                                        ELSE qs.statement_end_offset
                                      END - qs.statement_start_offset ) / 2 ) + 1) AS query_text ,
                        query_plan
               FROM     top_queries_by_hash tq
                        JOIN sys.dm_exec_query_stats qs ON tq.[query_hash] = qs.[query_hash]
                                                           AND tq.[max_plan_handle] = qs.[plan_handle]
                        CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS st
                        CROSS APPLY sys.dm_exec_query_plan(tq.max_plan_handle) AS qp
             )
    SELECT 
			execution_count AS [Total Executions] ,
			qt.query_hash,
			qt.query_plan_hash,		    
			number_of_plans AS [# Plans] ,
			distinct_plans as [Distinct Plans],
            total_worker_time AS [Total CPU] ,
            total_worker_time / execution_count AS [Avg CPU] ,
            CAST(ROUND(100.00 * total_worker_time / ( SELECT    SUM(total_worker_time)
                                                      FROM      sys.dm_exec_query_stats
                                                    ), 2) AS MONEY) AS [% CPU] ,
            CAST(ROUND(total_elapsed_time / 1000., 2) AS BIGINT) AS [Total Duration ms] ,
            CAST(total_elapsed_time / execution_count / 1000. AS BIGINT) AS [Avg Duration ms] ,
            CAST(ROUND(100.00 * total_elapsed_time / ( SELECT   SUM(total_elapsed_time)
                                                       FROM     sys.dm_exec_query_stats
                                                     ), 2) AS MONEY) AS [% Duration] ,
            total_logical_reads / execution_count AS [Avg Reads] ,
            total_logical_reads AS [Total Reads] ,
            CAST(ROUND(100.00 * total_logical_reads / ( SELECT  SUM(total_logical_reads)
                                                        FROM    sys.dm_exec_query_stats
                                                      ), 2) AS MONEY) AS [% Reads] ,
            CAST(ROUND(100.00 * execution_count / ( SELECT  SUM(execution_count)
                                                    FROM    sys.dm_exec_query_stats
                                                  ), 2) AS MONEY) AS [% Executions] ,
            CASE DATEDIFF(mi, min_creation_time, max_last_execution_time)
              WHEN 0 THEN 0
              ELSE CAST(( 1.00 * execution_count / DATEDIFF(mi, min_creation_time, max_last_execution_time) ) AS MONEY)
            END AS [Executions/Min] ,
            query_text AS [Query Text] ,
            query_plan AS [Estimated Plan]
    FROM    [top_queries_by_hash] tq
            LEFT JOIN [query_text_and_plans] qt ON tq.query_hash = qt.query_hash
    ORDER BY [Total Executions] DESC OPTION  ( RECOMPILE );
GO